<?php
/**
 * 
 * Extension to pScatter. Adds the posibillity to draw a timeline below the chart.
 * @version 0.1
 * @author onnerby
 *
 */
class pScatterTimeline extends pScatter {
	
	protected $timelineDefaultOptions	= array(
			'border' => array("R"=>255,"G"=>255,"B"=>255,"Alpha"=>100,"Weight"=>0),
			'row1'=>	array("R"=>142,"G"=>142,"B"=>142,"Alpha"=>100),
			'row2'=>	array("R"=>224,"G"=>224,"B"=>224,"Alpha"=>100),
			'rowtext1'=>	array("R"=>255,"G"=>255,"B"=>255,"Alpha"=>100),
			'rowtext2'=>	array("R"=>142,"G"=>142,"B"=>142,"Alpha"=>100),
			'levels'=> array(
				array(
					'period'=>'P0Y0M0D0H1M',
					'format'=>'Y-m-d H:i',
					'outputFormat'=>'H:i',
					'outputfullFormat'=>'H:i',
					'avgDuration'=>60,
				),
				array(
					'period'=>'P0Y0M0D1H0M',
					'format'=>'Y-m-d H:00',
					'outputFormat'=>'H',
					'outputfullFormat'=>'H',
					'avgDuration'=>3600,
				),
				array(
					'period'=>'P0Y0M1D',
					'format'=>'Y-m-d',
					'outputFormat'=>'D',
					'outputfullFormat'=>'d M',
					'avgDuration'=>86400,
				),
				array(
					'period'=>'P0Y0M1W',
					'format'=>'last monday',
					'outputFormat'=>'vW',
					'outputfullFormat'=>'vW Y',
					'avgDuration'=>604800,
				),
				array(
					'period'=>'P0Y1M',
					'format'=>'Y-m-01',
					'outputFormat'=>'M',
					'outputfullFormat'=>'M Y',
					'avgDuration'=>2628000,
				),
				array(
					'period'=>'P1Y',
					'format'=>'Y-01-01',
					'outputFormat'=>'Y',
					'outputfullFormat'=>'Y',
					'avgDuration'=>31536000,
				),
			),
		);
		
		
	private function _mergOptions(&$options,&$mergeTo){
		foreach($options as $key=>$value){
			if(is_array($options[$key]) && is_array($mergeTo[$key])){
				$this->_mergOptions($options[$key], $mergeTo[$key]);
			}else{
				$mergeTo[$key]	= $value;
			}
		}
	}
	
	
	function pScatterTimeline($pChartObject,$pDataObject){
		$this->pChartObject = $pChartObject;
		$this->pDataObject  = $pDataObject;
	}
	
	function drawScatterTimeline($options=array()){
		
		$this->_mergOptions($options,$this->timelineDefaultOptions);
		
		// Levels to show
		$showLevels	= array();
		$showLevelsMax	= 2;
		$cellMinWidth	= 20;
		$cellMinCount	= 2;
		$timelineHeight	= 20;
		$borderStyle	= $this->timelineDefaultOptions['border'];
		
		$AxisID	= 0;
		$data = $this->pDataObject->getData();
		
		$startDate	= $data["Axis"][$AxisID]["ScaleMin"];
		$endDate	= $data["Axis"][$AxisID]["ScaleMax"];
		$startX = $this->pChartObject->GraphAreaX1 + $data["Axis"][$AxisID]["Margin"];
		$endX 	= $this->pChartObject->GraphAreaX2 - $data["Axis"][$AxisID]["Margin"];
		$chartWidth	= $endX-$startX;
		$cellMaxCount	= $chartWidth/$cellMinWidth;
		
		$levels	= $this->timelineDefaultOptions['levels'];
		
		for($i=0;$i<count($levels) && count($showLevels)<$showLevelsMax;$i++){
			$levelInfo	= $levels[$i];
			if(($endDate-$startDate)/$levelInfo['avgDuration']<$cellMaxCount){
				// We have something that will fit in the chart.
				$showLevels[]	= $levelInfo;
			}
		}
		
		// Now we know what levels to show
		// Lets show them
		if ( !isset($data["Axis"][$AxisID]) ) { return(-1); }
		if ( $data["Axis"][$AxisID]["Identity"] == AXIS_X ){
			$y		= $this->pChartObject->GraphAreaY2 - $data["Axis"][$AxisID]["Margin"];
			
			// Draw background
			for($i=0;$i<count($showLevels);$i++){
       			$this->pChartObject->drawFilledRectangle($startX,$y+$i*$timelineHeight,$endX,$y+($i+1)*$timelineHeight,
       				$this->timelineDefaultOptions['row'.($i+1)]
       			);
			}
			
			// Draw border between
			for($i=0;$i<=count($showLevels);$i++){
       			$this->pChartObject->drawLine($startX,$y+$i*$timelineHeight,$endX,$y+$i*$timelineHeight,$borderStyle);
			}

			// Draw interval
			for($i=0;$i<count($showLevels);$i++){
				
				$last	= ($i==count($showLevels)-1);
				// Round the date to first occurens
				if(substr($showLevels[$i]['format'],0,1)=='Y'){
					$roundDate	= strtotime(date($showLevels[$i]['format'],$startDate));
				}else{
					$roundDate	= strtotime($showLevels[$i]['format'],$startDate);
				}

				$date	= new DateTime();
				$date->setTimestamp($roundDate);
				$interval	= new DateInterval($showLevels[$i]['period']);
				
				while($date->getTimestamp()<$endDate){
					
					$cellStartX  	= $this->getPosArray($date->getTimestamp(),$AxisID);
					$text	= $date->format($last ? $showLevels[$i]['outputfullFormat'] : $showLevels[$i]['outputFormat']);
					$date->add($interval);
					$cellEndX  	= $this->getPosArray($date->getTimestamp(),$AxisID);
					
					if($cellStartX>=$startX){
	       				$this->pChartObject->drawLine($cellStartX,$y+$i*$timelineHeight,$cellStartX,$y+($i+1)*$timelineHeight,$borderStyle);
					}
					$textX	= $cellStartX+($cellEndX-$cellStartX)*0.5;
					if($textX-$startX>0.5*$cellMinWidth && $textX<$endX-0.5*$cellMinWidth){
						// Draw text
						$this->pChartObject->drawText($textX,$y+($i+0.5)*$timelineHeight,$text,array('Align'=>TEXT_ALIGN_MIDDLEMIDDLE)+$this->timelineDefaultOptions['rowtext'.($i+1)]);
					}
				}
			}
		}
	}
	
	
}
